//===========================================================================
//
//  Filename  : main.cpp
//  Purpose   : Application and Controller for QtGantt
//  Copyright : Copyright (C) 2000 Matt J. Gumbley <matt@gumbley.demon.co.uk>
//  Original  : by Gunnstein Lye
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//===========================================================================

#include <stream.h>

#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qwidget.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qspinbox.h>
#include <qaccel.h>
#include <qfile.h>
#include <qprinter.h>

#include "gantt.h"
#include "main.h"
#include "project.h"

MainWidget::MainWidget( char *filenamearg, QWidget *parent, const char *name )
        : QWidget( parent, name )
	{
	int menuH; // determined after menubar creation
	int menuX; //      "       "      "        "
	int ctrlH = 25; // might be better the same height as the menu?
	frameT = 5; // thickness between window edge and frame edge
	setMinimumSize( 490, 300 );

	_menubar = new QMenuBar( this );
	QRect menuRect = _menubar->frameRect();
	menuH = menuRect.height();
	menuX = menuRect.x();
	frameY = menuH + ctrlH + (2 * frameT);
	
	frame = new QFrame( this, "ganttFrame" );
	frame->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
	frame->setGeometry( frameT, frameY, 
		width()-(frameT*2), height()-frameY-frameT );
	frameW = frame->frameWidth();

	project = new Project( filenamearg );

	gantt = new Gantt( project, this, "gantt" );
	gantt->setGeometry( frameT+frameW, frameY+frameW,
		width()-((frameT+frameW)*2), height()-((frameY+frameW)*2) );

	f5acc = new QAccel( this );    // Draws gantt when F5 is pressed
	f5acc->connectItem( f5acc->insertItem(Key_F5), this, SLOT( slotRefreshFile() ) );
	
	plusacc = new QAccel( this );    // Zooms in when + is pressed
	plusacc->connectItem( plusacc->insertItem(Key_Plus), this, SLOT( slotzoomIn() ) );
	minusacc = new QAccel( this );    // Zooms out when - is pressed
	minusacc->connectItem( minusacc->insertItem(Key_Minus), this, SLOT( slotzoomOut() ) );

	// CHANGE: split the gantt widget in two: a task name list and a task
	// detail widget, then separate the two with a QSplitter
	scrolltextpix = new QScrollBar( 0, 300, 1, 10, 150, QScrollBar::Horizontal, this, "scrolltextpix" );
	int stpW = 150;
	scrolltextpix->setGeometry( frameT, 5+menuH, stpW, ctrlH );
	connect( scrolltextpix, SIGNAL( valueChanged(int) ), gantt, SLOT(changetextpix(int)) );
	
	// zoom control
	spindaypix = new QSpinBox( MINDAYPIX, MAXDAYPIX, 1, this, "spindaypix" );
	spindaypix->setGeometry( frameT + stpW + 10 , 5+menuH, 50, ctrlH );
	spindaypix->setValue( 10 );
	connect( spindaypix, SIGNAL( valueChanged(int) ), gantt, SLOT(changedaypix(int)) );
	
	/* Create the menus */
	_filemenu = new QPopupMenu;
	CHECK_PTR( _filemenu );
	// All the menu slots are part of the MainWidget, which is the
	// controller. Project is the model; Gantt is one view.
	_filemenu->insertItem( "&New", this, SLOT( slotNewFile() ));
	_filemenu->insertItem( "&Open...", this, SLOT( slotOpenFile() ));
	_filemenu->insertItem( "&Print", this, SLOT( slotPrintFile() ));
	_filemenu->insertItem( "&Refresh", this, SLOT( slotRefreshFile() ));
	_filemenu->insertItem( "&Save", this, SLOT( slotSaveFile() ));
	_filemenu->insertItem( "&Quit", this, SLOT( slotQuit() ));

	_options = new QPopupMenu;
	CHECK_PTR( _options );
	weekdetailID = _options->insertItem( "&Week detail", this, SLOT( slottoggleweekdetail()) );
	isWeekdetail = FALSE;
	_options->setCheckable( TRUE ); 

	_viewmenu = new QPopupMenu;
	CHECK_PTR( _viewmenu );
	_viewmenu->insertItem( "Zoom in", this, SLOT( slotzoomIn()) );
	_viewmenu->insertItem( "Zoom out", this, SLOT( slotzoomOut()) );
	_viewmenu->insertSeparator();
	_viewmenu->insertItem( "&Options", _options );


	_helpmenu = new QPopupMenu;
	CHECK_PTR( _helpmenu );
	_helpmenu->insertItem( "&About QtGantt", this, SLOT( slotAbout() ));

	_menubar->insertItem( "&File", _filemenu );
	_menubar->insertItem( "&View", _viewmenu );
	_menubar->insertSeparator( );
	_menubar->insertItem( "&Help", _helpmenu );
	}


void MainWidget::slottoggleweekdetail()
{
	isWeekdetail = !isWeekdetail;
	_options->setItemChecked( weekdetailID, isWeekdetail );
	gantt->setweekdetail(isWeekdetail);
}

void MainWidget::slotzoomIn()
{
	int s = spindaypix->value();
	if (s < MAXDAYPIX ) {
		spindaypix->setValue( ++s );
		gantt->changedaypix(s);
	}
}

void MainWidget::slotzoomOut()
{
	int s = spindaypix->value();
	if (s > MINDAYPIX ) {
		spindaypix->setValue( --s );
		gantt->changedaypix(s);
	}
}

void MainWidget::resizeEvent( QResizeEvent * )
	{
	frame->resize( width()-(frameT*2), height()-frameY-frameT );
	gantt->resize( width()-((frameT+frameW)*2), height()-frameY-frameT-(frameW*2) );
	// Don't think we need to force a repaint, since resize will do it?
	//gantt->repaint();
	}


void MainWidget::slotNewFile()
{
	int t;
	if ( project->isChanged() )
	{
		switch (QMessageBox::information( this, "QtGantt",
				"This file contains unsaved work\n"
				"Do you want to save it before creating a new file?\n",
				"&Save", "&Don't save", "&Cancel", 0, 2))
		{
		case 0: // Save
			project->save();
			break;
		case 1: // Don't Save
			break;
		case 2: // Cancel
			return;
		}
	}

	// Reset and repaint
	project->reset();
	gantt->repaint(); // may have to be refresh
}

void MainWidget::slotOpenFile()
{
	if ( project->isChanged() )
	{
		switch (QMessageBox::information( this, "QtGantt",
				"This file contains unsaved work\n"
				"Do you want to save it before opening another file?\n",
				"&Save", "&Don't save", "&Cancel", 0, 2))
		{
		case 0: // Save
			project->save();
			break;
		case 1: // Don't Save
			break;
		case 2: // Cancel
			return;
		}
	}

	// Open file dialog; Open file (which resets before reading)
	QString filename( QFileDialog::getOpenFileName( 0, "*.gantt" ) );
	if ( !filename.isNull() )             // if cancel was not pressed
	{
		project->openFilename( filename );
		gantt->repaint(); // may have to be refresh
	}
}

void MainWidget::slotSaveFile()
{
	QMessageBox::information( this, "QtGantt",
				"Save not currently implemented\n");
}

void MainWidget::slotRefreshFile()
{
	if ( project->isChanged() )
	{
		switch (QMessageBox::information( this, "QtGantt",
				"This file has changed.\n"
				"Refreshing will abandon your changes.\n"
				"Do you still want to refresh?\n",
				"&Yes", "&No", 0, 1))
		{
		case 0: // Yes, reload
			break;
		case 1: // Don't reload
			return;
		}
	}
	project->refresh();
	gantt->repaint(); // may have to be refresh
	}

void MainWidget::slotPrintFile()
{
	QPrinter * printer = new QPrinter();
	gantt->print( printer );
}

void MainWidget::slotQuit()
{
	if ( project->isChanged() )
	{
		switch (QMessageBox::information( this, "QtGantt",
				"This file contains unsaved work\n"
				"Do you want to save it before quitting?\n",
				"&Save", "&Don't save", "&Cancel", 0, 2))
		{
		case 0: // Save
			project->save();
			break;
		case 1: // Don't Save
			break;
		case 2: // Cancel
			return;
		}
	}
	// emit to qApp? perhaps change this when I understand signals/slots
	// better
	qApp->quit();
}

void MainWidget::slotAbout()
{
	QMessageBox::information( this, "About QtGantt",
				"QtGantt - a project management system\n"
				"Copyright 2000 by Gunnstein Lye, Matt Gumbley\n"
				"and the gantt-devel@geekstuff.co.uk\n"
				"mailing list\n"
				);
}


int main( int argc, char **argv )
	{
	QApplication a( argc, argv );
	int i;
	char *filename = NULL;
	
	// There may be a "proper" C++/Qt way of handling the command line, but
	// I only know the C way, so here goes...
	for (i=1; i<argc; i++)
	{
		// printf("command line arg %d is [%s]\n", i, argv[i]);
		// Again, my C++ is not as good as my C... C++ string handling?
		// Handle options
		if (argv[i][0] == '-')
		{
			switch (argv[i][1])
			{
				case 'p' : // print to this file
					if (i == (argc-1))
					{
						cerr << "Must specify an output file with -p\n";
						exit(-1);
					}
					char *output = argv[++i];
					Project *project = new Project( filename
);
					Gantt *gantt = new Gantt( project, 0, "gantt" );
					QPrinter *printer = new QPrinter();
					gantt->printToFile( printer, output );
					exit(0);
					break;
			}
		}
		else
			filename = argv[i];
	}

	MainWidget w( a.argv()[1] );
	w.setGeometry( 0, 80, 630, 400 );
	w.setCaption( "QtGantt" );          // Sets the caption of the window
	
	a.setMainWidget( &w );
	w.show();
	return a.exec();
	}
